package be.business.connector.core.domain;

import be.business.connector.recipe.executor.domain.GetAllReservationsParam;
import be.business.connector.recipe.executor.domain.GetPrescriptionForExecutorResult;
import be.business.connector.recipe.executor.dto.ListOpenPrescriptionsResultWithExceptions;
import be.business.connector.recipe.executor.dto.ListReservationsResultWithExceptions;
import be.recipe.api.executor.ListMandatesByMandateHolder;
import be.recipe.services.core.MandateHolderType;
import be.recipe.services.executor.GetOpenPrescriptionForExecutor;
import be.recipe.services.executor.GetPrescriptionStatusParam;
import be.recipe.services.executor.GetPrescriptionStatusResult;
import be.recipe.services.executor.ListNotificationsItem;
import be.recipe.services.executor.ListOpenPrescriptionsParam;
import be.recipe.services.executor.ListOpenPrescriptionsResult;
import be.recipe.services.executor.ListRelationsParam;
import be.recipe.services.executor.ListRelationsResult;
import be.recipe.services.executor.ListReservationsParam;
import be.recipe.services.executor.ListReservationsResult;
import be.recipe.services.executor.ListRidsHistoryParam;
import be.recipe.services.executor.ListRidsHistoryResult;
import be.recipe.services.executor.ListRidsInProcessParam;
import be.recipe.services.executor.ListRidsInProcessResult;
import be.recipe.services.executor.PutRidsInProcessParam;
import be.recipe.services.executor.PutRidsInProcessResult;
import java.util.List;

public interface ExModuleV4
    extends ListMandatesByMandateHolder.Command<
        ListMandatesByMandateHolder, ListMandatesByMandateHolder.Response> {

  /**
   * Gets the {@link ListOpenPrescriptionsResult}.
   *
   * @param request a {@link ListOpenPrescriptionsParam}
   * @return the {@link ListOpenPrescriptionsResult}
   * @throws the integration module exception
   */
  ListOpenPrescriptionsResult getData(ListOpenPrescriptionsParam request);

  /**
   * Gets the {@link GetPrescriptionStatusResult}.
   *
   * @param gpsp the gpsp
   * @return the {@link GetPrescriptionStatusResult}
   * @throws the integration module exception
   */
  GetPrescriptionStatusResult getData(GetPrescriptionStatusParam gpsp);

  /**
   * Gets the {@link ListRidsHistoryResult}.
   *
   * @param request the {@link ListRidsHistoryParam}
   * @return the {@link ListPrescriptionHistoryParam}
   * @throws the integration module exception
   */
  ListRidsHistoryResult getData(ListRidsHistoryParam request);

  /**
   * Gets the {@link ListReservationsResult}.
   *
   * @param request the {@link GetAllReservationsParam}
   * @return the {@link ListReservationsResult}
   * @throws the integration module exception
   */
  ListReservationsResult getData(ListReservationsParam request);

  /**
   * Performs a getData(ListReservationsParam)- and decryptListReservationsResult-operation at once.
   *
   * @param request the {@link ListReservationsParam}
   * @return the decrypted {@link ListReservationsResult}
   * @throws the integration module exception
   */
  ListReservationsResult listReservationsDecrypted(ListReservationsParam request);

  ListReservationsResultWithExceptions listReservationsDecryptedWithExceptions(
          ListReservationsParam request);

  /**
   * Gets the {@link ListRidsInProcessResult}.
   *
   * @param request the {@link ListRidsInProcessParam}
   * @return the data
   * @return the {@link ListRidsInProcessResult}
   * @throws the integration module exception
   */
  ListRidsInProcessResult getData(ListRidsInProcessParam request);

  /**
   * Puts the {@link PutRidsInProcessResult}.
   *
   * @param request the {@link PutRidsInProcessParam}
   * @return the {@link PutRidsInProcessResult}
   * @throws the integration module exception
   */
  PutRidsInProcessResult putData(PutRidsInProcessParam request);

  /**
   * Gets the {@link GetPrescriptionForExecutorResult}
   * and marks it as delivered.
   *
   * @param rid the rid
   * @return the {@link
   *     GetPrescriptionForExecutorResult}
   * @throws the integration module exception
   */
  GetPrescriptionForExecutorResult getAndMarkAsDelivered(String rid);

  List<GetPrescriptionForExecutorResult> getAndMarkAsDelivered(List<String> rids);

  /**
   * Gets the {@link GetPrescriptionForExecutorResult}
   * and marks it as delivered.
   *
   * @param rid
   * @param mandateHolderId
   * @param mandateHolderType
   * @return the {@link
   *     GetPrescriptionForExecutorResult}
   * @throws the integration module exception
   */
  GetPrescriptionForExecutorResult getAndMarkAsDelivered(
          String rid, String mandateHolderId, MandateHolderType mandateHolderType);

  /**
   * Gets the {@link ListRelationsResult}.
   *
   * @param patientRelationParam the patient relation param
   * @return the {@link ListRelationsResult}
   * @throws the integration module exception
   */
  ListRelationsResult getData(ListRelationsParam patientRelationParam);

  /**
   * Decrypts a {@link GetOpenPrescriptionForExecutor}.
   *
   * @param getOpenPrescriptionForExecutor a the {@link GetOpenPrescriptionListParam}
   * @return a {@link GetOpenPrescriptionForExecutor}
   * @throws the integration module exception
   */
  GetOpenPrescriptionForExecutor decryptGetOpenPrescriptionForExecutor(
          final GetOpenPrescriptionForExecutor getOpenPrescriptionForExecutor);

  /**
   * Gets a prescription.
   *
   * @param rid the rid
   * @return the {@link GetPrescriptionForExecutorResult}
   * @throws the integration module exception
   */
  GetPrescriptionForExecutorResult getPrescription(String rid);

  GetPrescriptionForExecutorResult getPrescription(String rid, boolean alreadyDelivered);

  GetPrescriptionForExecutorResult getPrescription(
          String rid, String mandateHolderId, MandateHolderType mandateHolderType);

  /**
   * Gets a prescription.
   *
   * @param rid
   * @param mandateHolderId
   * @param mandateHolderType
   * @return the {@link GetPrescriptionForExecutorResult}
   * @throws the integration module exception
   */
  GetPrescriptionForExecutorResult getPrescription(
          String rid,
          String mandateHolderId,
          MandateHolderType mandateHolderType,
          boolean alreadyDelivered);

  /**
   * Marks a rid as archived.
   *
   * @param rid the rid
   * @throws the integration module exception
   */
  void markAsArchived(String rid);

  /**
   * Marks a rid as archived.
   *
   * @param rid the rid
   * @param mandateHolderId the mandateHolderId
   * @param mandateHolderType the mandateHolderType
   * @throws the integration module exception
   */
  void markAsArchived(String rid, String mandateHolderId, MandateHolderType mandateHolderType);

  /**
   * Marks a rid as delivered.
   *
   * @param rid the rid
   * @throws the integration module exception
   */
  void markAsDelivered(String rid);

  /**
   * Marks a rid as delivered.
   *
   * @param rid the rid
   * @param mandateHolderId the mandateHolderId
   * @param mandateHolderType the mandateHolderType
   * @throws the integration module exception
   */
  void markAsDelivered(String rid, String mandateHolderId, MandateHolderType mandateHolderType);

  /**
   * Marks a rid as undelivered.
   *
   * @param rid the rid
   * @throws the integration module exception
   */
  void markAsUndelivered(String rid);

  /**
   * Marks a rid as undelivered.
   *
   * @param rid the rid
   * @param mandateHolderId the mandateHolderId
   * @param mandateHolderType the mandateHolderType
   * @throws the integration module exception
   */
  void markAsUndelivered(String rid, String mandateHolderId, MandateHolderType mandateHolderType);

  /**
   * Revokes a rid.
   *
   * @param rid the rid
   * @param reason the reason
   * @throws the integration module exception
   */
  void revokePrescription(String rid, String reason);

  /**
   * Revokes a rid.
   *
   * @param rid the rid
   * @param mandateHolderId the mandateHolderId
   * @param mandateHolderType the mandateHolderType
   * @throws the integration module exception
   */
  void revokePrescription(
          String rid, String reason, String mandateHolderId, MandateHolderType mandateHolderType);

  /**
   * Returns a list of {@link ListNotificationsItem}.
   *
   * @param readFlag the read flag
   * @return the list
   * @throws the integration module exception
   */
  List<ListNotificationsItem> listNotifications(boolean readFlag);

  /**
   * Returns a list of {@link ListNotificationsItem}.
   *
   * @param readFlag
   * @param mandateHolderId
   * @param mandateHolderType
   * @return the list
   * @throws the integration module exception
   */
  List<ListNotificationsItem> listNotifications(
          boolean readFlag, String mandateHolderId, MandateHolderType mandateHolderType);

  /**
   * Creates a feedback.
   *
   * @param prescriberId the prescriber id
   * @param rid the rid
   * @param feedbackText the feedback text
   * @throws the integration module exception
   */
  void createFeedback(String prescriberId, String rid, byte[] feedbackText);

  /**
   * Creates a feedback.
   *
   * @param prescriberId
   * @param rid
   * @param feedbackText
   * @param mandateHolderId
   * @param mandateHolderType
   * @throws the integration module exception
   */
  void createFeedback(
          String prescriberId,
          String rid,
          byte[] feedbackText,
          String mandateHolderId,
          MandateHolderType mandateHolderType);

  /**
   * Decrypts a {@link ListOpenPrescriptionsResult}.
   *
   * @param listOpenPrescriptionsResult the encrypted {@link ListOpenPrescriptionsResult}
   * @return the decrypted {@link ListOpenPrescriptionsResult}
   * @throws the integration module exception
   */
  ListOpenPrescriptionsResult decryptListOpenPrescriptionsResult(
          ListOpenPrescriptionsResult listOpenPrescriptionsResult);

  ListOpenPrescriptionsResultWithExceptions decryptListOpenPrescriptionsResultWithExceptions(
          ListOpenPrescriptionsResult listOpenPrescriptionsResult);

  /**
   * Performs a ListOpenPrescription- & decryptListOpenPrescriptionsResult-operation at once.
   *
   * @param listOpenPrescriptionsResult the encrypted {@link ListOpenPrescriptionsResult}
   * @return the decrypted {@link ListOpenPrescriptionsResult}
   * @throws the integration module exception
   */
  ListOpenPrescriptionsResult listOpenPrescriptionsDecrypted(ListOpenPrescriptionsParam request);

  ListOpenPrescriptionsResultWithExceptions listOpenPrescriptionsWithExceptionsDecrypted(
          ListOpenPrescriptionsParam request);

  /**
   * Decrypts a {@link ListReservationsResult}.
   *
   * @param listReservationsResult the encrypted {@link ListReservationsResult}
   * @return the decrypted {@link ListReservationsResult}
   * @throws the integration module exception
   */
  ListReservationsResult decryptListReservationsResult(
          ListReservationsResult listReservationsResult);

  ListReservationsResultWithExceptions decryptListReservationsResultWithExceptions(
          ListReservationsResult listReservationsResult);

  /** Clears the session-map. */
  void clearSession();
}
